import 'package:dynatrace_flutter_plugin/src/agent/http/http_client_options.dart';
import 'package:dynatrace_flutter_plugin/src/agent/http/http_client_utils.dart';

import '../../agent/agent_impl.dart';
import '../../agent/interface/web_request_timing.dart';

import 'package:http/http.dart';

import '../util/logger.dart';

class DynatraceHttpClient extends BaseClient {
  final DynatraceImpl _dynatrace;
  final DynatraceHttpClientOptions _options;
  final Client _client;

  /// Constructor which wraps a [client]
  DynatraceHttpClient(this._dynatrace, this._options, {Client? client})
      : _client = client ?? Client();

  @override
  Future<StreamedResponse> send(BaseRequest request) async {
    StreamedResponse? response;
    String? header =
        await _dynatrace.getHTTPTagForWebRequest(request.url.toString());

    if (header != null && header.isNotEmpty) {
      request.headers['x-dynatrace'] = header;
    }

    if (request.headers.containsKey('x-dynatrace')) {
      WebRequestTiming timing = _dynatrace.createWebRequestTiming(
          request.headers['x-dynatrace']!, request.url.toString());
      int requestSize = HttpClientUtils.calcRequestSizeBytes(
          request, _options.requestHeaders);

      try {
        timing.startWebRequestTiming();
        response = await _client.send(request);
        int responseSize = HttpClientUtils.calcResponseSizeBytes(response);
        timing.stopWebRequestTimingWithSize(response.statusCode,
            response.reasonPhrase, requestSize, responseSize);
      } catch (error) {
        timing.stopWebRequestTiming(-1, error.toString());
        rethrow;
      }
      return response;
    } else {
      Logger().d(
          "DynatraceHttpClient send(${request.url.toString()}): The x-dynatrace header could not be set on the request!",
          logType: LogType.Info);
      return await _client.send(request);
    }
  }

  @override
  void close() => _client.close();
}
